@@ -4,15 +4,15 @@
 //! operations needs to be done quickly with these types.
 use std::fmt::{Debug, Display};
 use std::ops::{AddAssign, Mul};
-use std::slice::Iter;
+use std::slice::{Iter, IterMut};
 
-use num::Zero;
+use num_traits::Zero;
+use relp_num::NonZero;
 
 pub use dense::Dense as DenseVector;
 pub use sparse::Sparse as SparseVector;
 
 use crate::data::linear_algebra::SparseTuple;
-use crate::data::linear_algebra::traits::NotZero;
 
 mod dense;
 mod sparse;
@@ -37,16 +37,17 @@ pub trait Vector<F>: PartialEq + Display + Debug {
     /// Compute the inner product with a column vector from a matrix.
     fn sparse_inner_product<'a, H, G: 'a, V: Iterator<Item=&'a SparseTuple<G>>>(&self, column: V) -> H
     where
-        H: Zero + AddAssign<F>,
+        H: Zero + AddAssign<F> + Display + Debug,
+        G: Display + Debug,
         for<'r> &'r F: Mul<&'r G, Output=F>,
     ;
     /// Make a vector longer by one, by adding an extra value at the end of this vector.
-    fn push_value(&mut self, value: F) where F: NotZero;
+    fn push_value(&mut self, value: F) where F: NonZero;
     /// Set the value at an index.
     ///
     /// Depending on internal representation, this can be an expensive operation (for `SparseVector`
     /// 's, the cost depends on the (lack of) sparsity.
-    fn set(&mut self, index: usize, value: F) where F: NotZero;
+    fn set(&mut self, index: usize, value: F) where F: NonZero;
     /// Retrieve the value at an index.
     ///
     /// # Returns
@@ -56,7 +57,9 @@ pub trait Vector<F>: PartialEq + Display + Debug {
     /// Remove the items at the specified indices.
     fn remove_indices(&mut self, indices: &[usize]);
     /// Iterate over the internal values.
-    fn iter_values(&self) -> Iter<Self::Inner>;
+    fn iter(&self) -> Iter<Self::Inner>;
+    /// Iterate over the internal values mutably.
+    fn iter_mut(&mut self) -> IterMut<Self::Inner>;
     /// Number of items represented by the vector.
     fn len(&self) -> usize;
     /// Whether the vector is empty.
@@ -70,26 +73,22 @@ pub trait Vector<F>: PartialEq + Display + Debug {
 pub mod test {
     //! Contains also some helper methods to be used in other test modules.
 
-    use std::f64::EPSILON;
-
-    use num::{FromPrimitive, NumCast, ToPrimitive, Zero};
+    use num_traits::{FromPrimitive, NumCast};
+    use relp_num::{Field, FieldRef};
+    use relp_num::F;
+    use relp_num::NonZero;
 
     use crate::data::linear_algebra::traits::{SparseComparator, SparseElement};
     use crate::data::linear_algebra::vector::{DenseVector, SparseVector, Vector};
-    use crate::data::number_types::traits::{Field, FieldRef};
-    use crate::F;
 
     pub trait TestVector<F>: Vector<F> {
-        fn from_test_data<T: ToPrimitive + Zero>(data: Vec<T>) -> Self;
+        fn from_test_data<T: NonZero>(data: Vec<T>) -> Self where F: From<T>;
     }
-    impl<F: Field + FromPrimitive> TestVector<F> for DenseVector<F> {
+    impl<F: Field> TestVector<F> for DenseVector<F> {
         /// Create a `DenseVector` from the provided data.
-        fn from_test_data<T: ToPrimitive + Zero>(data: Vec<T>) -> Self {
-            Self {
-                data: data.into_iter()
-                    .map(|v| F::from_f64(v.to_f64().unwrap()).unwrap())
-                    .collect(),
-            }
+        fn from_test_data<T>(data: Vec<T>) -> Self where F: From<T> {
+            let len = data.len();
+            Self::new(data.into_iter().map(|v| F::from(v)).collect(), len)
         }
     }
     impl<F: SparseElement<C>, C: SparseComparator> TestVector<F> for SparseVector<F, C>
@@ -97,7 +96,7 @@ pub mod test {
         F: Field + FromPrimitive,
     {
         /// Create a `SparseVector` from the provided data.
-        fn from_test_data<T: ToPrimitive + Zero>(data: Vec<T>) -> Self {
+        fn from_test_data<T: NonZero>(data: Vec<T>) -> Self where F: From<T> {
             debug_assert_ne!(data.len(), 0);
 
             let size = data.len();
@@ -105,8 +104,8 @@ pub mod test {
             Self::new(
                 data.into_iter()
                     .enumerate()
-                    .filter(|(_, v)| !v.is_zero())
-                    .map(|(i, v)| (i, F::from_f64(v.to_f64().unwrap()).unwrap()))
+                    .filter(|(_, v)| v.is_not_zero())
+                    .map(|(i, v)| (i, F::from(v)))
                     .collect(),
                 size,
             )
@@ -123,7 +122,7 @@ pub mod test {
             debug_assert!(data.is_sorted_by_key(|&(i, _)| i));
             debug_assert_ne!(len, 0);
             debug_assert!(data.len() <= len);
-            debug_assert!(data.iter().all(|&(_, v)| v.to_f64().unwrap().abs() > EPSILON));
+            debug_assert!(data.iter().all(|&(_, v)| v.to_f64().unwrap().abs() > f64::EPSILON));
 
             Self::new(
                 data.into_iter().map(|(i, v)| {
@@ -135,12 +134,12 @@ pub mod test {
     }
 
     /// A test vector used in tests.
-    fn get_test_vector<F: Field + FromPrimitive, V: TestVector<F>>() -> V {
+    fn get_test_vector<F: Field + From<u8> + NonZero, V: TestVector<F>>() -> V {
         V::from_test_data(vec![0, 5, 6])
     }
 
     /// Test
-    fn push_value<F: Field + FromPrimitive, V: TestVector<F>>() where for<'r> &'r F: FieldRef<F> {
+    fn push_value<F: Field + FromPrimitive + From<u8> + NonZero, V: TestVector<F>>() where for<'r> &'r F: FieldRef<F> {
         let mut v = get_test_vector::<F, V>();
         let len = v.len();
         let new_v = F!(1);
@@ -151,7 +150,7 @@ pub mod test {
     }
 
     /// Test
-    fn get_set<F: Field + FromPrimitive, V: TestVector<F>>() {
+    fn get_set<F: Field + FromPrimitive + From<u8> + NonZero, V: TestVector<F>>() {
         let mut v = get_test_vector::<F, V>();
 
         // Getting a nonzero value
@@ -174,21 +173,22 @@ pub mod test {
     }
 
     /// Test
-    fn out_of_bounds_get<F: Field + FromPrimitive, V: TestVector<F>>() {
+    #[allow(unused_must_use)]
+    fn out_of_bounds_get<F: Field + NonZero + From<u8> + FromPrimitive, V: TestVector<F>>() {
         let v = get_test_vector::<F, V>();
 
         &v.get(400);
     }
 
     /// Test
-    fn out_of_bounds_set<F: Field + FromPrimitive, V: TestVector<F>>() {
+    fn out_of_bounds_set<F: Field + From<u8> + FromPrimitive + NonZero, V: TestVector<F>>() {
         let mut v = get_test_vector::<F, V>();
 
         v.set(400, F!(45));
     }
 
     /// Test
-    fn len<F: Field + FromPrimitive, V: TestVector<F>>() {
+    fn len<F: Field + From<u8> + NonZero, V: TestVector<F>>() {
         let v = get_test_vector::<F, V>();
 
         assert_eq!(v.len(), 3);
@@ -196,14 +196,13 @@ pub mod test {
 
     #[cfg(test)]
     mod dense_vector {
-        use num::traits::FromPrimitive;
-        use num::Zero;
+        use num_traits::{FromPrimitive, Zero};
+        use relp_num::{F, R32};
+        use relp_num::{Rational32, Rational64};
+        use relp_num::Field;
 
-        use crate::{F, R32};
         use crate::data::linear_algebra::vector::{DenseVector, Vector};
         use crate::data::linear_algebra::vector::test::{get_set, get_test_vector, len, out_of_bounds_get, out_of_bounds_set, push_value, TestVector};
-        use crate::data::number_types::rational::{Rational32, Rational64};
-        use crate::data::number_types::traits::Field;
 
         type T = Rational32;
 
@@ -287,12 +286,12 @@ pub mod test {
 
     #[cfg(test)]
     mod sparse_vector {
-        use num::FromPrimitive;
+        use num_traits::FromPrimitive;
+        use relp_num::R32;
+        use relp_num::Rational32;
 
         use crate::data::linear_algebra::vector::{SparseVector, Vector};
         use crate::data::linear_algebra::vector::test::{get_set, get_test_vector, len, out_of_bounds_get, out_of_bounds_set, push_value, TestVector};
-        use crate::data::number_types::rational::Rational32;
-        use crate::R32;
 
         type T = Rational32;
 
