@@ -1,6 +1,6 @@
 use std::collections::HashMap;
 
-use num::FromPrimitive;
+use relp_num::R32;
 
 use crate::data::linear_algebra::matrix::{ColumnMajor, Order};
 use crate::data::linear_algebra::vector::{DenseVector, Vector};
@@ -10,8 +10,6 @@ use crate::data::linear_program::elements::LinearProgramType;
 use crate::data::linear_program::general_form::{GeneralForm, RangedConstraintRelation, Variable};
 use crate::data::linear_program::general_form::presolve::updates::Changes;
 use crate::data::linear_program::general_form::RemovedVariable;
-use crate::data::number_types::rational::Rational32;
-use crate::R32;
 
 /// A simple problem that should not be presolved (further).
 ///
